using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using BlackStar;

namespace BlackStarTest
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class GameMain : Game
    {
        GraphicsDeviceManager graphics;
        GUI gui;
        AppConsole console;
        LoginForm loginForm;

        public GameMain()
        {
            Content.RootDirectory = "Content";
            graphics = new GraphicsDeviceManager(this);
            //graphics.IsFullScreen = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            gui = new GUI(this, "GUI\\GUI",
                                "GUI\\Themes\\Default\\Theme",
                                "GUI\\KeymapDefault");

            // since this is a windows app, we can use the clipboard, if we wanted 
            // to run this on the xbox or zune, would skip this step.
            gui.clipboard.TextGet += new ClipboardRequest(clipboard_TextGet);
            gui.clipboard.TextSet += new ClipboardSet(clipboard_TextSet);

            gui.ComponentCreated += new ComponentEvent(gui_ComponentCreated);
            gui.KeyDown += new KeyEvent(gui_KeyDown);

            base.Initialize();
        }

        void clipboard_TextSet(string value)
        {
            System.Windows.Forms.Clipboard.SetText(value);
        }

         
        string clipboard_TextGet()
        {
            if (System.Windows.Forms.Clipboard.ContainsText())
            { return System.Windows.Forms.Clipboard.GetText(); }
            else
            { return string.Empty; }
        }

        void gui_KeyDown(KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Keys.OemTilde:
                    Component Console = gui.FindComponent("Console");
                    Console.Visible = !Console.Visible;
                    break;
            }
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            console = new AppConsole(gui, "Console");
            loginForm = new LoginForm(this, gui, "FormLogin");

            gui.FindComponent("FormLogin\\TextboxUsername").TakeFocus();
        }

        protected void gui_ComponentCreated(Component sender)
        {
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            base.Draw(gameTime);
        }
    }
}
